<?php
/*
 * Plugin GMap
 * Golocalisation des objets SPIP et insertion de cartes
 *
 * Auteur :
 * Fabrice ALBERT
 * (c) 2009-2011 - licence GNU/GPL
 *
 * Balise GEOMAP :
 *  Insertion d'une carte depuis un squelette ou un modle.
 *
 * Paramtres :
 *	map			: id de la carte cible (1 si omis, ce qui est le dfaut de toutes les balises)
 *	id_article/id_rubrique...	: objet  partir duquel est affiche la carte. S'il n'est pas golocalis, la carte n'est pas affiche, s'il l'est, la carte est centre dessus, et il sert de base  la recherche des marqueurs
 *	markers		: dfinition des marqueurs (local : seul(s) le(s) marqueur(s) de l'objet sont affichs; childs : les marqueurs de l'objet et de ses descendants de niveau 1 sont affichs; recursive : les marqueurs de l'objet et de tous ses descendants sont affichs; query : la liste des marqueurs provient d'une requte ajax)
 *	query		: nom du fichier ajax de requte (si markers=query)
 *  viewport	: objet qui sert de rfrence pour le positionnement et le zoom initial de la carte
 *	latitude	: latitude du centre de la carte
 *	longitude	: longitude du centre de la carte
 *	zoom		: zoom initial de la carte
 *	fond		: fond de carte par dfaut(plan / satellite / mixte / physic / earth)
 * ... une multitude d'autres paramtres dpendants plus ou moins de l'implmentation de la carte (Google Maps V2/V3, autre ?)
 *
 * Exemples : 
 *	#GEOMAP{id_document=140}
 *	#GEOMAP{id_document} (dans une boucle qui contient id_document dans son contexte)
 *
 */

if (!defined("_ECRIRE_INC_VERSION")) return;

include_spip('inc/gmap_geoloc');
include_spip('balise/gmap_balises');

// Balise GEOMAP : renvoie les informations sur le marqueur associ  un point sur un objet
function balise_GEOMAP($p)
{
	return _gmap_calculer_balise($p, 'GEOMAP');
}
function balise_GEOMAP_stat($args, $filtres)
{
	$params = _gmap_calculer_balise_params($args);
	if (isset($params['map']))
		$mapId = $params['map'];
	else
		$mapId = $GLOBALS['nextMapID']++;
	$params['map'] = $GLOBALS['currentMapID'] = $mapId;
	return gmap_geomap($params);
}
// Pas de partie dynamique : on calcule tout avant le cache

// Ajout de la carte
function gmap_geomap($params)
{
	$code = "";
	$mapId = $params['map'];

	// Ajout d'une div si ncessaire
	$bCreateDiv = (isset($params['creatediv']) && $params['creatediv']) ? true : false;
	if ($bCreateDiv)
	{
		$width = $params['width'] ? $params['width'] : "100%";
		$height = $params['height'] ? $params['height'] : "400px";
		$code .= '
<div style="display: block; position: relative; width: '.$width.'; height: '.$height.';">
';
	}

	// Gnrer le rsultat
	$code .= gmap_ajoute_carte_public($params['objet'], $params['id_objet'], $mapId, $params);
	
	// Fermer la div
	if ($bCreateDiv)
		$code .= '
</div>
';
	
	return $code;
}

?>