<?php
/*
 * Plugin GMap
 * Golocalisation des objets SPIP et insertion de cartes
 *
 * Auteur :
 * Fabrice ALBERT
 * (c) 2009-2011 - licence GNU/GPL
 *
 * Outils pour la gestion des balises et balises statiques (les plus simples, pour
 * les balises plus compliques au niveau des arguments, le modle des balises
 * dynamiques de SPIP est plus appropri car il prend en charge le dcodage des
 * paramtres.
 *
 */

if (!defined("_ECRIRE_INC_VERSION")) return;

include_spip('inc/gmap_db_utils');
include_spip('inc/gmap_geoloc');


// Mcanisme d'allocation automatique des numros de carte
$GLOBALS['currentMapID'] = 1;
$GLOBALS['nextMapID'] = 1;


/* Outils pour grer facilement les balises dynamiques dans le contexte de GMap
 * Les besoins sont un peu toujours les mmes : on prend du contexte ou des
 * paramtres un id d'objet golocalisable et les balises acceptent d'autres paramtres
 */

$GLOBALS['context_query'] = array(
	'objet', 'id_objet',
	'id_document',
	'id_article',
	'id_breve',
	'id_rubrique',
	'id_secteur',
	'id_mot',
	'id_auteur',
	'type_point'
);

// Au dpart : lancer la balise dynamique
function _gmap_calculer_balise($p, $nom)
{
	return calculer_balise_dynamique(
			$p,		//  le nud AST pour la balise
			$nom,	//  le nom de la balise
			$GLOBALS['context_query']); //  les lments utilisables de l'environnement	
}

// Dcomposition d'un paramtre en paire clef/valeur, selon qu'il y a ou non
// un signe '=' dans la chaine...
function _gmap_split_param($param)
{
	$key = null;
	$value = null;
	if (($pos = strpos($param, "=")) !== false)
	{
		$key = substr($param, 0, $pos);
		$value = trim(substr($param, $pos+1));
		if (!$value || !strlen($value)) // cas de l'appel depuis les modles o l'on a '$key='
			$key = null;
	}
	else
	{
		$key = $param;
		$value = true;
	}
	if (is_string($key))
		$key = trim($key);
	if (!$key || !strlen($key))
		$value = null;
	return array($key, $value);
}

// Dans la partie statique, dcoder les paramtres
// Renvoie un tableau de paramtres nomms contenant :
// - toujours "objet" et "id_objet"
// - "type_point", une chaine vide s'il n'est pas dfini
// - tous les autres paramtres passs  la balise
function _gmap_calculer_balise_params($args, $bStrictParams = false)
{
	// Commencer par dcoder le paramtres explicites, donc ceux qui sont aprs ce
	// qui a t demand dans la pile
	$params = array();
	for ($index = count($GLOBALS['context_query']); $index < count($args); $index++)
	{
		list($key, $value) = _gmap_split_param($args[$index]);
		if (!$key)
			continue;
		$params[$key] = $value;
	}
	
	// Si une requte est prcise, dsactiver la recherche d'un point de 
	// rfrence  tout prix : la personne qui a donn la requte doit 
	// savoir quels paramtres il faut passer
	if (isset($params['query']))
		$bStrictParams = true;
	
	// Inialiser les paramtres implicites
	$objet = null;
	$id_objet = null;
	$type_point = null;
	
	// Traiter le cas "type_point" en premier, il simplifiera la suite
	if (isset($params["type_point"]))
	{
		if ($params["type_point"] === true)
			$type_point = $args[array_search("type_point", $GLOBALS['context_query'], true)];
		else
			$type_point = $params["type_point"];
		unset($params["type_point"]);
	}
	
	// Traiter le cas de "objet" dont peut dpendre le type d'objet choisit
	if (isset($params["objet"]))
	{
		if (is_string($params["objet"]))
			$objet = $params["objet"];
		unset($params["objet"]);
	}
	
	// Finalement, traiter le cas de id_objet qui est prioritaire, mais
	// seulement si $objet est dj dfini
	if (isset($params["id_objet"]))
	{
		if ($objet && is_string($params["id_objet"]))
			$id_objet = $params["id_objet"];
		unset($params["id_objet"]);
	}
	
	// L, on a nettoy les paramtres passs manuellement ($params) de "objet", "id_objet" et
	// "type_point", si on n'a trouv aucun objet, on va essayer d'en trouv un dans la pile
	
	// Si on n'a pas de id_objet, mais un objet dsign, le chercher
	if (!$id_objet && is_string($objet))
	{
		$idName = 'id_'.$objet;
		if (isset($params[$idName]) && is_string($params[$idName]))
			$id_objet = $params[$idName];
		else if ($index = array_search($idName, $GLOBALS['context_query'], true))
			$id_objet = $args[$index];
	}
	
	// Si on n'a pas de id_objet, parcourir les paramtres explicites pour voir si
	// un type d'objet est dsign (c'est le cas le plus courant : on a mis dans les
	// paramtre {id_rubrique} ou {id_rubrique=XX})
	if (!$id_objet)
	{
		// Rechercher le premier paramtre qui correspond  un des paramtres implicites
		foreach ($params as $key => $value)
		{
			if (($key === "objet") || ($key === "id_objet") || ($key === "type_point"))
				continue;
			if (in_array($key, $GLOBALS['context_query'], true))
			{
				if (preg_match("/^id_([a-z]*)$/i", $key, $matches))
				{
					$objet = $matches[1];
					if ($value === true)
						$id_objet = $args[array_search($key, $GLOBALS['context_query'], true)];
					else
						$id_objet = $value;
					break;
				}
			}
		}
	}
	
	// Si on n'a toujours rien, parcourir les paramtres implicites dans l'ordre
	// de prfrence
	// Ici, il serait mieux de prendre la boucle la plus proche, mais je ne sais pas
	// faire a dans une balise dynamique : il faudrait dcoder les codes PHP renvoys
	// pour les valeurs afin de voir lequel a l'indice le plus proche
	if (!$id_objet && ($bStrictParams !== true))
	{
		foreach ($GLOBALS['context_query'] as $index => $name)
		{
			if (($name === "objet") || ($name === "id_objet") || ($name === "type_point"))
				continue;
			$value = $args[$index];
			if (isset($value) && is_string($value) && strlen($value))
			{
				if (preg_match("/^id_([a-z]*)$/i", $name, $matches))
				{
					$objet = $matches[1];
					$id_objet = $value;
					break;
				}
			}
		}
	}
	
	// Nettoyer de l'objet trouv, puis rechercher dans la pile les autres
	foreach ($params as $name => $value)
	{
		// Supprimer si c'est l'objet qu'on a trouv
		if ($name === 'id_'.$objet)
			unset($params[$name]);
			
		// Si aucune valeur explicite n'est donne, tenter de reprendre dans la
		// pile
		else if (!is_string($value))
		{
			if ($index = array_search($name, $GLOBALS['context_query'], true))
				$params[$name] = $args[$index];
			if (!$params[$name] || !strlen($params[$name]))
				unset($params[$name]);
		}
	}
		
	// Remettre l'objet et le type de point
	if (isset($objet) && isset($id_objet) && is_string($objet) && strlen($objet))
	{
		$params['objet'] = $objet;
		$params['id_objet'] = $id_objet;
	}
	if (isset($type_point) && is_string($type_point) && strlen($type_point))
		$params['type_point'] = $type_point;
	
	return $params;
}


/*
 * Balises statiques
 */

// Balise URL_FICHIER : recherche d'un fichier par find_in_path.
function balise_URL_FICHIER($p)
{
	// Rcuprer l'argument
	$filePath = interprete_argument_balise(1, $p);
	
	// Retour
	if (!$filePath)
		$p->code = "";
	else
		$p->code = "find_in_path(".$filePath.")";
	$p->interdire_scripts = false;
	return $p;
}

// Balise URL_FICHIER_DEF : recherche par find_in_path avec un dossier alternatif
function balise_URL_FICHIER_DEF($p)
{
	// Rcuprer l'argument
	$filePath = interprete_argument_balise(1, $p);
	$folder = interprete_argument_balise(2, $p);
	
	// Retour
	if (!$filePath)
		$p->code = "";
	else
		$p->code = "_gmap_find_in_path(".$filePath.", ".$folder.")";
	$p->interdire_scripts = false;
	return $p;
}

// Balise GEOCAPABILITY : test des capacits d'une implmentation de carte
function balise_GEOCAPABILITY($p)
{
	// Rcuprer l'argument
	$capability = interprete_argument_balise(1, $p);
	
	// Retour
	if (!$capability)
		$p->code = "";
	else
		$p->code = "gmap_teste_capability(".$capability.")";
	$p->interdire_scripts = false;
	return $p;
}

?>
