<?php
/*
 * Plugin GMap
 * Golocalisation des objets SPIP et insertion de cartes
 *
 * Auteur :
 * Fabrice ALBERT
 * (c) 2009-2011 - licence GNU/GPL
 *
 * Boucles et critres introduits par le plugin :
 * - GEOTEST : test si un objet est golocalis ou contient des objets golocaliss
 *
 */

if (!defined("_ECRIRE_INC_VERSION")) return;

include_spip("inc/gmap_config_utils");
include_spip('inc/gmap_db_utils');


/*
 * La seule boucle  crer est GEOTEST, qui a un comportement particulier.
 * Le plugin introduit d'autres boucles mais qui sont prises en compte directement
 * par SPIP puisqu'elles sont lies  des tables.
 * GEOTEST est un peu particulire car son rsultat ne dpend pas des champs SQL
 * mais d'un algorithme qui compte le nombre de points sur un objet et ses 
 * descendants.
 * Dans SPIP, une boucle correspond  une table dans une base de donnes
 * (cf. ecrire/public/compiler.php),  moins qu'elle ne soit dclare optionelle
 * (BOUCLE_(TABLE ?)) ou qu'on dfinisse un nouveau connecteur de base de donnes.
 * Pour simplifier, GEOTEST est associe  gmap_points_liens (cf. base/gmap_tables.php)
 */

 
/*
 * Outils pour grer les boucles
 */

// Ordre de prfrence des champs id
// Utilis quand l'objet sur lequel on fait les requtes n'est pas dfini
$GLOBALS['preferred_ids'] = array(
	'id_mot',
	'id_auteur',
	'id_document',
	'id_article',
	'id_breve',
	'id_rubrique',
	'id_secteur'
);

// Recherche d'une boucle de nom prdfini dans la pile
// Les fonction du core de SPIP :
// - index_pile : recherche un champ dans la pile de boucle, mais ce champs doit tre dfinis dans les tables
// - rindex_pile : recherche une boucle qui a un critre donn
// ... ne sont pas adaptes  ce que je veux faire ici : chercher un nom de boucle particulier car il intgre le champs que
// je veux rcuprer.
// Donc je refais une fonction boucle_index_pile qui recherche une boucle d'un nom donn.
// $idb : id de boucle de dpart (boucle actuelle)
// $type_boucle : type de la boucle (ARTICLES, GEOTEST)
// $nom_champ : nom du champ recherch
// &$boucles : tableaux des boucles
// $explicite : id d'une boucle de dmarrage explicite (autre que $idb)
// $defaut : si TRUE et que rien n'est trouv dans les boucles, on prend les paramtres
function _gmap_boucle_index_pile($idb, $type_boucle, $nom_champ, &$boucles, $explicite='', $defaut = TRUE)
{
	$i = 0; // nombre de niveaux remonts
	$type_boucle = strtolower($type_boucle);
	$nom_champ = strtolower($nom_champ);
	
	// Remonter d'abord jusqu' la boucle de nom $explicite
	if (strlen($explicite))
	{
		while (($idb !== $explicite) && ($idb !==''))
		{
			$i++;
			$idb = $boucles[$idb]->id_parent;
		}
	}

	// Puis remonter jusqu' une boucle sur la table voulue
	while (isset($boucles[$idb]))
	{
		// Si c'est la bonne boucle, renvoyer le champs dans la pile
		// ATTENTION : on n'a aucun moyen de tester si le champ existe bien
		// c'est un pari sur la cohrence du squelette !
		if ($boucles[$idb]->type_requete === $type_boucle)
			return '@$Pile[$SP' . ($i ? "-$i" : "") . '][\'' . $nom_champ . '\']';

		// Sinon on remonte d'un cran
		$idb = $boucles[$idb]->id_parent;
		$i++;
	}

	// esperons qu'il y sera
	if ($defaut)
		return('@$Pile[0][\''. $nom_champ . '\']');
	else
		return NULL;
}

// Recherche de la dfinition d'un objet dans les boucles englobantes (ou dans les paramtres de la page)
// Renvoie un tableau associatif :
// 'objet' => type d'objet
// 'id_objet' => id de l'objet, NULL s'il n'est pas dfini
function _gmap_find_boucle_object($objetDef, &$boucles, $idb, $explicite='', $defaut = TRUE)
{
	// On commence sur la boucle locale
	$i = 0;
	
	// Si une autre boucle est dsigne explicitement, l'utiliser comme point de dpart
	if (strlen($explicite))
	{
		while (($idb !== $explicite) && ($idb !==''))
		{
			$i++;
			$idb = $boucles[$idb]->id_parent;
		}
	}

	// Remonter les boucles
	// Sur chaque niveau, rechercher un id qui pourrait nous convenir
	while (isset($boucles[$idb]))
	{
		// Si un nom est donn, rechercher seulement celui-l
		if (strlen($objetDef['objet']))
		{
			list ($t, $c) = index_tables_en_pile($idb, 'id_'.$objetDef['objet'], $boucles);
			if ($t)
			{
				// Ajouter le champ  la requte
				if (!in_array($t, $boucles[$idb]->select))
					$boucles[$idb]->select[] = $t;
				
				// Stocker le code qui permet de le rcuprer
				$objetDef['id_objet'] = '$Pile[$SP' . ($i ? "-$i" : "") . '][\'' . $c . '\']';
				return $objetDef;
			}
		}
		
		// Sinon tout essayer
		else
		{
			// Parcours des IDs connus
			foreach ($GLOBALS['preferred_ids'] as $idName)
			{
				// Les tables contiennent-elles ce champ ?
				list ($t, $c) = index_tables_en_pile($idb, $idName, $boucles);
				if ($t)
				{
					// Ajouter le champ  la requte
					if (!in_array($t, $boucles[$idb]->select))
						$boucles[$idb]->select[] = $t;
					
					// Stocker le code qui permet de le rcuprer
					$objetDef['objet'] = substr($idName, strlen("id_"));
					$objetDef['id_objet'] = '$Pile[$SP' . ($i ? "-$i" : "") . '][\'' . $c . '\']';
					return $objetDef;
				}
			}
		}

		// Sinon on remonte d'un cran
		$idb = $boucles[$idb]->id_parent;
		$i++;
	}

	// On n'a rien trouv, rcuprer dans la requte
	if ($defaut)
	{
		// Si un type est dfini, on le prend des paramtres en esprant qu'il sera l
		if (strlen($objetDef['objet']))
		{
			$objetDef['id_objet'] = '@$Pile[0][\'id_'. $objetDef['objet'] . '\']';
			return $objetDef;
		}
		// Sinon on peut essayer de scanner les paramtres...
		else
		{
			// Parcours des IDs connus
			foreach ($GLOBALS['preferred_ids'] as $idName)
			{
				if (_request($idName))
				{
					$objetDef['objet'] = substr($idName, strlen("id_"));
					$objetDef['id_objet'] = _request($idName);
					return $objetDef;
				}
			}
		}
	}
	
	// Rien  faire...
	return NULL;
}

// Rcupration des paramtres d'une boucle sous une forme pratique
// Retour :
// Tableau associatif $nom-du-paramtre => $valeur, s'il y en a une
function _gmap_boucle_params(&$boucle)
{
	// La premire source d'information est le tableau criteres :
	// criteres[index] :
	// 	op : operation, soit un oprateur (=,!=,>...), soit le texte du critre s'il n'y a pas d'oprateur (id_article)
	// 	not :
	// 	exclus :
	// 	param : tableau des paramtres de l'opration, si c'est un oprateur
	// 		param[index] tableau : (je ne sais pas  quoi sert ce niveau)
	// 			param[index][0] texte :
	// 				param[index][0]->texte : texte du paramtre
	// 				param[index][0]->avant :
	// 				param[index][0]->apres :
	//
	// Le tableau param fournit aussi des infos : les '?' ajouts derrire les critres
	// param[index] :
	// 	param[index][0] : je n'en connais pas l'utilit
	// 	param[index][1] : tableau
	// 		param[index][1][0] : objet texte
	// 		param[index][1][0]->texte : texte complet du paramtre, sauf dans le cas d'un oprateur (id_article ?)
	// 		param[index][1][0]->avant :
	// 		param[index][1][0]->apres :
	
	// Vrifier qu'il y a des paramtres
	if (count($boucle->criteres) == 0)
		return NULL;
		
	// Init retour
	$params = NULL;
	
	// Parcours des critres
	foreach ($boucle->criteres as $crit)
	{
		// Si c'est un oprateur, il a des paramtres
		if ($crit->param && (count($crit->param) > 0))
		{
			// Pour ce qui est de GMAP, on ne supporte que l'oprayeur d'galit...
			if ($crit->op != '=')
			{
				spip_log("Boucle ".$boucle->id_boucle." : critre '".$crit->op."' non gr.", "gmap");
				continue;
			}
				
			// Vrifier les donnes : on doit avoir deux paramtres
			if (!$crit->param || (count($crit->param) != 2))
			{
				spip_log("Boucle ".$boucle->id_boucle." : critre '=' illisible (plus ou moins de deux paramtres)", "gmap");
				continue;
			}
			// chacun des deux paramtres doit contenir un tableau dont le premier lment est un texte
			if ((count($crit->param[0]) != 1) || (count($crit->param[1]) != 1) ||
				($crit->param[0][0]->type !== 'texte') || ($crit->param[1][0]->type !== 'texte'))
			{
				spip_log("Boucle ".$boucle->id_boucle." : critre '=' illisible (paramtres non textuels)", "gmap");
				continue;
			}

			// Ajouter dans le tableau
			if (!$params)
				$params = array();
			$params[$crit->param[0][0]->texte] = $crit->param[1][0]->texte; // on retient le dernier qui est prcis
		}
		else // c'est un critre seul
		{
			// Ajouter dans le tableau
			if (!$params)
				$params = array();
			if (!isset($params[$crit->param[0]->texte])) // on retient le premier, qui devait tre plus prcis
				$params[$crit->op] = TRUE;
		}
	}
	
	// Ici, on n'a pas l'info si un '?' est ajout derrire le critre, pour a, il faut parcourir le tableau params...
	foreach ($boucle->param as $param)
	{
		// Vrifier les donnes 
		if ((count($param) < 2) || (count($param[1]) != 1) ||
		    ($param[1][0]->type !== 'texte'))
			continue;
		
		// Voir s'il y a un ? dans le texte
		$crit = $param[1][0]->texte;
		if (($pos = strpos($crit, '?')) === FALSE)
			continue;
		$crit = trim(substr($crit, 0, $pos));
		
		// Si un critre du mme nom existe dj et qu'il n'avait pas de valeur, le mettre  '?'
		if ($params[$crit] === TRUE)
			$params[$crit] = '?';
	}
	
	return $params;
}

// Recherche de la dfinition d'un objet dans les paramtres
// Renvoie un tableau associatif :
// 'objet' => type d'objet
// 'id_objet' => id de l'objet, NULL s'il n'est pas dfini
function _gmap_find_boucle_param_objet($params, $idb, &$boucles)
{
	// Vrifications
	if (!$params || (count($params) == 0))
		return NULL;

	// On commence par chercher des dfinitions prcises des objets (sans '?')
	// Partir des IDs les plus prcis vers les plus englobants, parmi les objets qui
	// peuvent tre golocaliss
	foreach ($GLOBALS['preferred_ids'] as $idName)
	{
		if (!$params[$idName])
			continue;

		// Sauter les dfinitions hasardeuses
		if ($params[$idName] === '?')
			continue
		
		// On a un objet prcis, l'utiliser
		$objetDef = array();
		$objetDef['objet'] = substr($idName, strlen("id_"));
		if (is_string($params[$idName]))
			$objetDef['id_objet'] = $params[$idName];
		else
			$objetDef['id_objet'] = NULL;
			
		// Rechercher cet objet dans la pile
		$objetDef = _gmap_find_boucle_object($objetDef, $boucles, $idb);
		
		return $objetDef;
	}
	
	// Les paramtres ne contiennent pas de critre prcis, seulement des
	// critres avec '?'.
	// Dans ce cas, on les parcours du plus prcis au plus large et on va regarder
	// si la pile de boucle contient quelque chose.
	$objetDef = array();
	foreach ($GLOBALS['preferred_ids'] as $idName)
	{
		if (!$params[$idName] || ($params[$idName] !== '?'))
			continue;
		$objetDef['objet'] = substr($idName, strlen("id_"));
		$objetDef['id_objet'] = NULL;
		$objetDef = _gmap_find_boucle_object($objetDef, $boucles, $idb, '', FALSE);
		if (strlen($objetDef['id_objet']))
			return $objetDef;
	}
	
	// Dernire chance, rcuprer dans la requte
	foreach ($GLOBALS['preferred_ids'] as $idName)
	{
		if (!$params[$idName] || ($params[$idName] !== '?'))
			continue;
		if (_request($idName))
		{
			$objetDef['objet'] = substr($idName, strlen("id_"));
			$objetDef['id_objet'] = _request($idName);
			return $objetDef;
		}
	}
	
	return NULL;
}

// Petite bidouille sur la mthode SPIP public/criteres.php/critere_par_joint pour ne pas forcment mettre les quotes
function _gmap_critere_par_joint($table, $champ, &$boucle, $idb, $bQuotes = true)
{
	$t = array_search($table, $boucle->from);
	if (!$t) $t = trouver_jointure_champ($champ, $boucle);
	return !$t ? '' : ($bQuotes ? ("'" . $t . '.' . $champ . "'") : $t . '.' . $champ);
}



/*
 * Boucle GEOTEST et critres associs
 */

// Boucle spcialise pour tester la prsence d'info gographique sur
// un objet ou sur ses descendants
function boucle_GEOTEST_dist($id_boucle, &$boucles)
{
	// Passer le pipeline, au cas o quelqu'un voudrait nous modifier la boucle...
	$boucles[$id_boucle] = pipeline('post_boucle', $boucles[$id_boucle]);

	// Rcuprer la boucle
	$boucle = &$boucles[$id_boucle];
	
	// Dbut de la boucle
	$corps = '
	$SP++;
	$Numrows["'.$id_boucle.'"]["compteur_boucle"] = 0;
	$Numrows["'.$id_boucle.'"]["total"] = 0;
	$Numrows["'.$id_boucle.'"]["grand_total"] = 0;
	$boucle->numrows = FALSE;
	$t0 ="";';
	
	// D'abord tester si le plugin est activ
	// On ne teste pas ici si l'objet est golocalisable : mme s'il a t golocalis par le pass, et ne l'est plus,
	// on considre que les positions saisies peuvent tre affiches sur une carte. En fait, ce sont aux
	// squelettes de dterminer si une carte doit tre affiche sur un objet ou non, pas au paramtrage
	// global.
	if (gmap_est_actif())
	{
		// Rcuprer les paramtres et l'objet sur lequel on fait la requte
		$params = _gmap_boucle_params($boucle);
		$objetDef = _gmap_find_boucle_param_objet($params, $id_boucle, $boucles); // id_article, id_rubrique...
		
		// Rcuprer les autres paramtres
		$recursive = ($params['recursif'] ? true : false);
		$visible = ($params['visible'] ? true : false);
	
		// Si une carte peut-tre affiche, ajouter le contenu de la boucle
		if ($objetDef)
		{
			$corps .= '
	if (gmap_est_objet_geo("'.$objetDef['objet'].'", '.$objetDef['id_objet'].', '.($visible?'true':'false').', '.($recursive?'true':'false').'))
	{
		$boucle->numrows = TRUE;
		$Numrows["'.$id_boucle.'"]["compteur_boucle"]++;
		$t0 .=' . $boucle->return . ';
	}';
		}
	}
	
	// Fin de boucle et retour
	$corps .= '
	$Numrows["'.$id_boucle.'"]["total"] = $Numrows["'.$id_boucle.'"]["grand_total"] = $Numrows["'.$id_boucle.'"]["compteur_boucle"];';
	return $corps 
		. "\n	return \$t0;";
}
function critere_GMAP_POINTS_LIENS_recursif_dist($idb, &$boucles, $crit)
{
}
function critere_GMAP_POINTS_LIENS_visible_dist($idb, &$boucles, $crit)
{
}


/*
 * Critres sur GEOPOINTS
 */
 
// Critre {visible} permettant de ne slectionner que les points visibles
function critere_GMAP_POINTS_visible_dist($idb, &$boucles, $crit)
{
	include_spip('public/criteres');
	$boucle = &$boucles[$idb];
	$visible = critere_par_joint('spip_gmap_types', 'visible', $boucle, $idb);
	$boucle->where[] = array("'='", $visible, $crit->not ? "sql_quote('non')" : "sql_quote('oui')");
}

// Critre {meilleur} permettant de slectionner le meilleur point pour un objet
/* Requte de test (slectionne le meilleur point de tous les objets) :
SELECT *
 FROM spip_gmap_points
 JOIN spip_gmap_points_liens ON spip_gmap_points.id_point = spip_gmap_points_liens.id_point
 JOIN spip_gmap_types ON spip_gmap_types.id_type_point = spip_gmap_points.id_type_point
 WHERE spip_gmap_types.priorite =
	(SELECT MIN(typesSub.priorite) AS maxpri
		FROM spip_gmap_points AS pointsSub
		 JOIN spip_gmap_points_liens AS liensSub ON pointsSub.id_point = liensSub.id_point
		 JOIN spip_gmap_types AS typesSub ON typesSub.id_type_point = pointsSub.id_type_point
		WHERE liensSub.objet = spip_gmap_points_liens.objet AND liensSub.id_objet = spip_gmap_points_liens.id_objet
		GROUP BY liensSub.objet, liensSub.id_objet)
*/
function critere_GMAP_POINTS_meilleur_dist($idb, &$boucles, $crit)
{
	include_spip('public/criteres');
	$boucle = &$boucles[$idb];
	$priorite = critere_par_joint('spip_gmap_types', 'priorite', $boucle, $idb);
	$objet = _gmap_critere_par_joint('spip_gmap_points_liens', 'objet', $boucle, $idb, false);
	$id_objet = _gmap_critere_par_joint('spip_gmap_points_liens', 'id_objet', $boucle, $idb, false);
	$subSelect =
		"SELECT MIN(typesSub.priorite)"
		." FROM spip_gmap_points AS pointsSub"
		." JOIN spip_gmap_points_liens AS liensSub ON pointsSub.id_point = liensSub.id_point"
		." JOIN spip_gmap_types AS typesSub ON typesSub.id_type_point = pointsSub.id_type_point"
		." WHERE liensSub.objet = ".$objet." AND liensSub.id_objet = ".$id_objet
		." GROUP BY liensSub.objet, liensSub.id_objet";
	$boucle->where[] = array("'='", $priorite, "'(".$subSelect.")'");
}


?>
